Package: anndataR
Title: AnnData interoperability in R
Version: 1.1.3
Authors@R: c(
    person("Robrecht", "Cannoodt", , "rcannood@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-3641-729X", github = "rcannood")),
    person("Luke", "Zappia", , "luke@lazappi.id.au", role = "aut",
           comment = c(ORCID = "0000-0001-7744-8565", github = "lazappi")),
    person("Martin", "Morgan", , "mtmorgan.bioc@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0002-5874-8148", github = "mtmorgan")),
    person("Louise", "Deconinck", , "louise.deconinck@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0001-8100-6823", github = "LouiseDck")),
    person("Danila", "Bredikhin", , "danila.bredikhin@embl.de", role = "ctb",
           comment = c(ORCID = "0000-0001-8089-6983", github = "gtca")),
    person("Isaac", "Virshup", role = "ctb",
           comment = c(ORCID = "0000-0002-1710-8945", github = "ivirshup")),
    person("Brian", "Schilder", , "brian_schilder@alumni.brown.edu", role = "ctb",
           comment = c(ORCID = "0000-0001-5949-2191", github = "bschilder")),
    person("Chananchida", "Sang-aram", role = "ctb",
           comment = c(ORCID = "0000-0002-0922-0822", github = "csangara")),
    person("Artür", "Manukyan", role = "ctb",
           comment = c(ORCID = "0000-0002-0441-9517", github = "Artur-man")),
    person("Hugo", "Gruson", role = "ctb",
           comment = c(ORCID = "0000-0002-4094-1476", github = "Bisaloo")),
    person("Data Intuitive", , , "info@data-intuitive.com", role = "fnd"),
    person("Chan Zuckerberg Initiative", role = "fnd"),
    person("scverse consortium", role = "spn")
  )
Description: Bring the power and flexibility of AnnData to the R
    ecosystem, allowing you to effortlessly manipulate and analyse your
    single-cell data. This package lets you work with backed h5ad and zarr
    files, directly access various slots (e.g. X, obs, var), or convert
    the data into SingleCellExperiment and Seurat objects.
License: MIT + file LICENSE
URL: https://anndataR.scverse.org/, https://github.com/scverse/anndataR
BugReports: https://github.com/scverse/anndataR/issues
Depends: R (>= 4.5.0)
Imports: cli, lifecycle, Matrix, methods, purrr, R6 (>= 2.4.0),
        reticulate (>= 1.41.1), rlang, stats
Suggests: BiocFileCache, BiocStyle, knitr, processx, rhdf5 (>= 2.52.1),
        Rarr (>= 1.11.12), rmarkdown, S4Vectors, Seurat, SeuratObject,
        SingleCellExperiment, spelling, SummarizedExperiment, testthat
        (>= 3.0.0), vctrs, withr, yaml
VignetteBuilder: knitr
biocViews: SingleCell, DataImport, DataRepresentation
Config/Needs/website: pkgdown, tibble, knitr, rprojroot, stringr,
        readr, purrr, dplyr, tidyr, rmarkdown
Config/testthat/edition: 3
Encoding: UTF-8
Roxygen: list(markdown = TRUE, r6 = TRUE)
RoxygenNote: 7.3.3
Language: en-GB
git_url: https://git.bioconductor.org/packages/anndataR
git_branch: devel
git_last_commit: b050bbe
git_last_commit_date: 2026-04-24
Repository: Bioconductor 3.23
Date/Publication: 2026-04-24
NeedsCompilation: no
Packaged: 2026-04-24 20:16:45 UTC; biocbuild
Author: Robrecht Cannoodt [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3641-729X>, github: rcannood),
  Luke Zappia [aut] (ORCID: <https://orcid.org/0000-0001-7744-8565>,
    github: lazappi),
  Martin Morgan [aut] (ORCID: <https://orcid.org/0000-0002-5874-8148>,
    github: mtmorgan),
  Louise Deconinck [aut] (ORCID: <https://orcid.org/0000-0001-8100-6823>,
    github: LouiseDck),
  Danila Bredikhin [ctb] (ORCID: <https://orcid.org/0000-0001-8089-6983>,
    github: gtca),
  Isaac Virshup [ctb] (ORCID: <https://orcid.org/0000-0002-1710-8945>,
    github: ivirshup),
  Brian Schilder [ctb] (ORCID: <https://orcid.org/0000-0001-5949-2191>,
    github: bschilder),
  Chananchida Sang-aram [ctb] (ORCID:
    <https://orcid.org/0000-0002-0922-0822>, github: csangara),
  Artür Manukyan [ctb] (ORCID: <https://orcid.org/0000-0002-0441-9517>,
    github: Artur-man),
  Hugo Gruson [ctb] (ORCID: <https://orcid.org/0000-0002-4094-1476>,
    github: Bisaloo),
  Data Intuitive [fnd],
  Chan Zuckerberg Initiative [fnd],
  scverse consortium [spn]
Maintainer: Robrecht Cannoodt <rcannood@gmail.com>
