% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ZarrAnnData.R
\name{as_ZarrAnnData}
\alias{as_ZarrAnnData}
\title{Convert an \code{AnnData} to an \code{ZarrAnnData}}
\usage{
as_ZarrAnnData(
  adata,
  file,
  compression = c("none", "gzip", "blosc", "zstd", "lzma", "bz2", "zlib", "lz4"),
  mode = c("w-", "r", "r+", "a", "w", "x")
)
}
\arguments{
\item{adata}{An \code{AnnData} object to be converted to \code{\link{ZarrAnnData}}}

\item{file}{The file name (character) of the \code{.zarr} file}

\item{compression}{The compression algorithm to use when writing the
Zarr file. Can be one of \code{"none"}, \code{"gzip"}, \code{"blosc"}, \code{"zstd"},
\code{"lzma"}, \code{"bz2"}, \code{"zlib"} or \code{"lz4"}. Defaults to \code{"none"}.}

\item{mode}{The mode to open the Zarr file:
\itemize{
\item \code{a} creates a new file or opens an existing one for read/write
\item \code{r} opens an existing file for reading
\item \verb{r+} opens an existing file for read/write
\item \code{w} creates a file, truncating any existing ones
\item \verb{w-}/\code{x} are synonyms, creating a file and failing if it already exists
}}
}
\value{
A \code{\link{ZarrAnnData}} object with the same data as the input \code{AnnData}
object.
}
\description{
Convert another \code{AnnData} object to an \code{\link{ZarrAnnData}} object
}
\seealso{
Other object converters: 
\code{\link{as_AnnData}()},
\code{\link{as_HDF5AnnData}()},
\code{\link{as_InMemoryAnnData}()},
\code{\link{as_ReticulateAnnData}()},
\code{\link{as_Seurat}()},
\code{\link{as_SingleCellExperiment}()},
\code{\link{reticulate-helpers}}
}
\concept{object converters}
\keyword{internal}
