% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ZarrAnnData.R
\name{ZarrAnnData}
\alias{ZarrAnnData}
\title{ZarrAnnData}
\value{
A \code{ZarrAnnData} object
}
\description{
Implementation of a Zarr-backed \code{AnnData} object. This class provides an
interface to a Zarr file and minimal data is stored in memory until it is
requested by the user. It is primarily designed as an intermediate object
when reading/writing Zarr files but can be useful for accessing parts of
large files.

See \link{AnnData-usage} for details on creating and using \code{AnnData} objects.
}
\seealso{
\link{AnnData-usage} for details on creating and using \code{AnnData} objects

Other AnnData classes: 
\code{\link{AbstractAnnData}},
\code{\link{AnnDataView}},
\code{\link{HDF5AnnData}},
\code{\link{InMemoryAnnData}},
\code{\link{ReticulateAnnData}}
}
\concept{AnnData classes}
\section{Super class}{
\code{\link[anndataR:AbstractAnnData]{anndataR::AbstractAnnData}} -> \code{ZarrAnnData}
}
\section{Active bindings}{
\if{html}{\out{<div class="r6-active-bindings">}}
\describe{
\item{\code{X}}{See \link{AnnData-usage}}

\item{\code{layers}}{See \link{AnnData-usage}}

\item{\code{obsm}}{See \link{AnnData-usage}}

\item{\code{varm}}{See \link{AnnData-usage}}

\item{\code{obsp}}{See \link{AnnData-usage}}

\item{\code{varp}}{See \link{AnnData-usage}}

\item{\code{obs}}{See \link{AnnData-usage}}

\item{\code{var}}{See \link{AnnData-usage}}

\item{\code{obs_names}}{See \link{AnnData-usage}}

\item{\code{var_names}}{See \link{AnnData-usage}}

\item{\code{uns}}{See \link{AnnData-usage}}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-ZarrAnnData-new}{\code{ZarrAnnData$new()}}
\item \href{#method-ZarrAnnData-n_obs}{\code{ZarrAnnData$n_obs()}}
\item \href{#method-ZarrAnnData-n_vars}{\code{ZarrAnnData$n_vars()}}
\item \href{#method-ZarrAnnData-obs_keys}{\code{ZarrAnnData$obs_keys()}}
\item \href{#method-ZarrAnnData-var_keys}{\code{ZarrAnnData$var_keys()}}
\item \href{#method-ZarrAnnData-layers_keys}{\code{ZarrAnnData$layers_keys()}}
\item \href{#method-ZarrAnnData-obsm_keys}{\code{ZarrAnnData$obsm_keys()}}
\item \href{#method-ZarrAnnData-varm_keys}{\code{ZarrAnnData$varm_keys()}}
\item \href{#method-ZarrAnnData-obsp_keys}{\code{ZarrAnnData$obsp_keys()}}
\item \href{#method-ZarrAnnData-varp_keys}{\code{ZarrAnnData$varp_keys()}}
\item \href{#method-ZarrAnnData-uns_keys}{\code{ZarrAnnData$uns_keys()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="anndataR" data-topic="AbstractAnnData" data-id="as_HDF5AnnData"><a href='../../anndataR/html/AbstractAnnData.html#method-AbstractAnnData-as_HDF5AnnData'><code>anndataR::AbstractAnnData$as_HDF5AnnData()</code></a></span></li>
<li><span class="pkg-link" data-pkg="anndataR" data-topic="AbstractAnnData" data-id="as_InMemoryAnnData"><a href='../../anndataR/html/AbstractAnnData.html#method-AbstractAnnData-as_InMemoryAnnData'><code>anndataR::AbstractAnnData$as_InMemoryAnnData()</code></a></span></li>
<li><span class="pkg-link" data-pkg="anndataR" data-topic="AbstractAnnData" data-id="as_ReticulateAnnData"><a href='../../anndataR/html/AbstractAnnData.html#method-AbstractAnnData-as_ReticulateAnnData'><code>anndataR::AbstractAnnData$as_ReticulateAnnData()</code></a></span></li>
<li><span class="pkg-link" data-pkg="anndataR" data-topic="AbstractAnnData" data-id="as_Seurat"><a href='../../anndataR/html/AbstractAnnData.html#method-AbstractAnnData-as_Seurat'><code>anndataR::AbstractAnnData$as_Seurat()</code></a></span></li>
<li><span class="pkg-link" data-pkg="anndataR" data-topic="AbstractAnnData" data-id="as_SingleCellExperiment"><a href='../../anndataR/html/AbstractAnnData.html#method-AbstractAnnData-as_SingleCellExperiment'><code>anndataR::AbstractAnnData$as_SingleCellExperiment()</code></a></span></li>
<li><span class="pkg-link" data-pkg="anndataR" data-topic="AbstractAnnData" data-id="as_ZarrAnnData"><a href='../../anndataR/html/AbstractAnnData.html#method-AbstractAnnData-as_ZarrAnnData'><code>anndataR::AbstractAnnData$as_ZarrAnnData()</code></a></span></li>
<li><span class="pkg-link" data-pkg="anndataR" data-topic="AbstractAnnData" data-id="print"><a href='../../anndataR/html/AbstractAnnData.html#method-AbstractAnnData-print'><code>anndataR::AbstractAnnData$print()</code></a></span></li>
<li><span class="pkg-link" data-pkg="anndataR" data-topic="AbstractAnnData" data-id="shape"><a href='../../anndataR/html/AbstractAnnData.html#method-AbstractAnnData-shape'><code>anndataR::AbstractAnnData$shape()</code></a></span></li>
<li><span class="pkg-link" data-pkg="anndataR" data-topic="AbstractAnnData" data-id="write_h5ad"><a href='../../anndataR/html/AbstractAnnData.html#method-AbstractAnnData-write_h5ad'><code>anndataR::AbstractAnnData$write_h5ad()</code></a></span></li>
<li><span class="pkg-link" data-pkg="anndataR" data-topic="AbstractAnnData" data-id="write_zarr"><a href='../../anndataR/html/AbstractAnnData.html#method-AbstractAnnData-write_zarr'><code>anndataR::AbstractAnnData$write_zarr()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ZarrAnnData-new"></a>}}
\if{latex}{\out{\hypertarget{method-ZarrAnnData-new}{}}}
\subsection{Method \code{new()}}{
\code{ZarrAnnData} constructor
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ZarrAnnData$new(
  file,
  X = NULL,
  obs = NULL,
  var = NULL,
  layers = NULL,
  obsm = NULL,
  varm = NULL,
  obsp = NULL,
  varp = NULL,
  uns = NULL,
  shape = NULL,
  mode = c("a", "r", "r+", "w", "w-", "x"),
  compression = c("none", "gzip", "blosc", "zstd", "lzma", "bz2", "zlib", "lz4")
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{file}}{The file name (character) of the \code{.zarr} file. If this file
already exits, other arguments must be \code{NULL}.}

\item{\code{X}}{See the \code{X} slot in \link{AnnData-usage}}

\item{\code{obs}}{See the \code{obs} slot in \link{AnnData-usage}}

\item{\code{var}}{See the \code{var} slot in \link{AnnData-usage}}

\item{\code{layers}}{See the \code{layers} slot in \link{AnnData-usage}}

\item{\code{obsm}}{See the \code{obsm} slot in \link{AnnData-usage}}

\item{\code{varm}}{See the \code{varm} slot in \link{AnnData-usage}}

\item{\code{obsp}}{See the \code{obsp} slot in \link{AnnData-usage}}

\item{\code{varp}}{See the \code{varp} slot in \link{AnnData-usage}}

\item{\code{uns}}{See the \code{uns} slot in \link{AnnData-usage}}

\item{\code{shape}}{Shape tuple (e.g. \code{c(n_obs, n_vars)}). Can be provided if
both \code{X} or \code{obs} and \code{var} are not provided.}

\item{\code{mode}}{The mode to open the Zarr file. See \code{\link[=as_ZarrAnnData]{as_ZarrAnnData()}} for
details}

\item{\code{compression}}{The compression algorithm to use. See
\code{\link[=as_ZarrAnnData]{as_ZarrAnnData()}} for details}
}
\if{html}{\out{</div>}}
}
\subsection{Details}{
The constructor creates a new Zarr \code{AnnData} interface object. This can
either be used to either connect to an existing \code{.zarr} file or to
create a new one. If any additional slot arguments are set an existing
file will be overwritten.
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ZarrAnnData-n_obs"></a>}}
\if{latex}{\out{\hypertarget{method-ZarrAnnData-n_obs}{}}}
\subsection{Method \code{n_obs()}}{
See the \code{n_obs} field in \link{AnnData-usage}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ZarrAnnData$n_obs()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ZarrAnnData-n_vars"></a>}}
\if{latex}{\out{\hypertarget{method-ZarrAnnData-n_vars}{}}}
\subsection{Method \code{n_vars()}}{
See the \code{n_vars} field in \link{AnnData-usage}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ZarrAnnData$n_vars()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ZarrAnnData-obs_keys"></a>}}
\if{latex}{\out{\hypertarget{method-ZarrAnnData-obs_keys}{}}}
\subsection{Method \code{obs_keys()}}{
See \link{AnnData-usage}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ZarrAnnData$obs_keys()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ZarrAnnData-var_keys"></a>}}
\if{latex}{\out{\hypertarget{method-ZarrAnnData-var_keys}{}}}
\subsection{Method \code{var_keys()}}{
See \link{AnnData-usage}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ZarrAnnData$var_keys()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ZarrAnnData-layers_keys"></a>}}
\if{latex}{\out{\hypertarget{method-ZarrAnnData-layers_keys}{}}}
\subsection{Method \code{layers_keys()}}{
See \link{AnnData-usage}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ZarrAnnData$layers_keys()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ZarrAnnData-obsm_keys"></a>}}
\if{latex}{\out{\hypertarget{method-ZarrAnnData-obsm_keys}{}}}
\subsection{Method \code{obsm_keys()}}{
See \link{AnnData-usage}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ZarrAnnData$obsm_keys()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ZarrAnnData-varm_keys"></a>}}
\if{latex}{\out{\hypertarget{method-ZarrAnnData-varm_keys}{}}}
\subsection{Method \code{varm_keys()}}{
See \link{AnnData-usage}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ZarrAnnData$varm_keys()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ZarrAnnData-obsp_keys"></a>}}
\if{latex}{\out{\hypertarget{method-ZarrAnnData-obsp_keys}{}}}
\subsection{Method \code{obsp_keys()}}{
See \link{AnnData-usage}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ZarrAnnData$obsp_keys()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ZarrAnnData-varp_keys"></a>}}
\if{latex}{\out{\hypertarget{method-ZarrAnnData-varp_keys}{}}}
\subsection{Method \code{varp_keys()}}{
See \link{AnnData-usage}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ZarrAnnData$varp_keys()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ZarrAnnData-uns_keys"></a>}}
\if{latex}{\out{\hypertarget{method-ZarrAnnData-uns_keys}{}}}
\subsection{Method \code{uns_keys()}}{
See \link{AnnData-usage}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ZarrAnnData$uns_keys()}\if{html}{\out{</div>}}
}

}
}
